@@ -1,8 +1,10 @@
 #ifndef SIMDJSON_PADDING_STRING_H
 #define SIMDJSON_PADDING_STRING_H
 #include "simdjson/portability.h"
-#include <memory>
 #include <cstring>
+#include <memory>
+
+namespace simdjson {
 // low-level function to allocate memory with padding so we can read passed the
 // "length" bytes safely. if you must provide a pointer to some data, create it
 // with this function: length is the max. size in bytes of the string caller is
@@ -63,5 +65,6 @@ private:
   size_t viable_size;
   char *data_ptr;
 };
+} // namespace simdjson
 
 #endif
